/***************************************************************************
 *   Copyright (C) 2006-2007 Marc Boris Duerner                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "unicode.h"

namespace zim
{

namespace
{
const unsigned short ctype_lookup1[68] =
{
  0,    104,  169,  257,  385,  402,  402,  498,
  626,  626,  677,  755,  402,  402,  402,  402,
  402,  402,  402,  402,  402,  402,  402,  402,
  402,  402,  402,  402,  402,  402,  402,  402,
  402,  402,  402,  402,  402,  402,  402,  402,
  402,  402,  402,  402,  402,  402,  402,  402,
  402,  402,  402,  402,  402,  402,  402,  402,
  883,  402,  402,  402,  1011, 1011, 1011, 1012,
  1011, 1011, 1011, 1012
};


const unsigned short ctype_lookup2[1140] =
{
  0,     127,   255,   382,   502,   582,   710,   837,
  965,   1091,  1219,  1339,  1467,  1580,  1708,  1836,
  1886,  1886,  2013,  2140,  2267,  2394,  2521,  2647,
  2774,  2900,  3026,  3152,  3279,  3406,  3534,  3662,
  3790,  3886,  4014,  4069,  4197,  4318,  4431,  4556,
  4683,  4684,  4684,  4684,  4703,  4831,  4959,  5087,
  5215,  5343,  5471,  5588,  1886,  1886,  1886,  1886,
  1886,  1886,  5716,  1886,  5844,  5950,  6078,  6206,
  6334,  6462,  6590,  6714,  6830,  6830,  6958,  7083,
  7211,  7311,  7439,  7512,  7640,  7768,  7895,  8013,
  7439,  7439,  6830,  8138,  6830,  6830,  8266,  1886,
  1886,  1886,  1886,  1886,  1886,  8394,  7439,  8424,
  8552,  8657,  8785,  8898,  9026,  9154,  7439,  7439,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  9282,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  9410,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  9538,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  9666,  9794,  9794,  9794,  9794,  9794,  9794,  9794,
  9794,  9794,  9794,  9794,  9794,  9794,  9794,  9794,
  9794,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  4684,  4684,  10050, 1886,  10178,
  10256, 4684,  4684,  10339, 10451, 10579, 10697, 10825,
  10938, 11066, 11194, 11322, 1886,  1886,  1886,  11450,
  11578, 11706, 11804, 1886,  1886,  1886,  1886,  1886,
  1886,  11932, 1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  7439,  12060, 12188, 12313, 1886,  1886,
  12441, 1886,  12569, 12695, 12819, 12941, 12965, 13083,
  13153, 13223, 1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  4684,  4684,  4684,  4684,  4684,  4684,
  4684,  4684,  13351, 1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  4684,  4684,  4684,  4684,  13408,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  13535, 1886,  13663, 13679, 1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  1886,  1886,  1886,  1886,  1886,
  1886,  1886,  1886,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  9922,  9922,  9922,  9922,  9922,
  9922,  9922,  9922,  13807
};


// Planes upper
const unsigned short upper_plane[68]=
{
  0,    74,   74,   76,   203,  74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74
};

// Blocks upper
const unsigned short upper_block[331]=
{
  0,     123,   251,   379,   499,   627,   686,   770,
  894,   1020,  1147,  1268,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1402,  1508,  1636,  1764,
  1275,  1275,  1880,  1275,  1275,  1275,  1275,  1275,
  1275,  2008,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  32,    1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  2114,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275,  1275,  1275,  1275,  1275,  1275,
  1275,  1275,  1275
};


const short upper_data[2242]=
{
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,    -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,
  -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,    0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   743,   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,    -32,   -32,   -32,   -32,   -32,
  -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,
  -32,   -32,    0,    -32,   -32,   -32,   -32,   -32,   -32,   -32,    121,   0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -232,   0,    -1,     0,
  -1,     0,    -1,     0,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,     0,    -1,     0,    -1,     0,    -1,    -300,   0,     0,     0,    -1,     0,
  -1,     0,     0,    -1,     0,     0,     0,    -1,     0,     0,     0,     0,     0,    -1,     0,     0,
   97,    0,     0,     0,    -1,     0,     0,     0,     0,     130,   0,     0,    -1,     0,    -1,     0,
  -1,     0,     0,    -1,     0,     0,     0,     0,    -1,     0,     0,    -1,     0,     0,     0,    -1,
   0,    -1,     0,     0,    -1,     0,     0,     0,    -1,     0,     56,    0,     0,     0,     0,     0,
  -1,    -2,     0,    -1,    -2,     0,    -1,    -2,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,    -79,    0,    -1,     0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,     0,    -1,    -2,     0,
  -1,     0,     0,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,     0,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,    -1,     0,    -1,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,    -210,  -206,   0,    -205,  -205,   0,    -202,   0,    -203,   0,
   0,     0,     0,    -205,   0,     0,    -207,   0,     0,     0,     0,    -209,  -211,   0,     0,     0,
   0,     0,    -211,   0,     0,    -213,   0,     0,    -214,   0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,    -218,   0,     0,    -218,   0,     0,     0,     0,    -218,   0,    -217,  -217,   0,
   0,     0,     0,     0,     0,    -219,   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     84,    0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,    -38,   -37,
  -37,   -37,    0,    -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,
  -32,   -32,   -32,   -32,   -31,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -64,   -63,
  -63,    0,    -62,   -57,    0,     0,     0,    -47,   -54,    0,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,    -86,   -80,    7,     0,     0,    -96,    0,     0,    -1,     0,     0,    -1,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,    -32,   -32,
  -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,
  -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -32,   -80,   -80,
  -80,   -80,   -80,   -80,   -80,   -80,   -80,   -80,   -80,   -80,   -80,   -80,   -80,   -80,    0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,     0,
   0,     0,     0,     0,     0,     0,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,     0,    -1,     0,
  -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,     0,     0,    -1,     0,     0,     0,     0,     0,     0,    -1,     0,    -1,     0,
  -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,    -48,   -48,   -48,   -48,
  -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,
  -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48,    0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,     0,     0,     0,     0,    -59,
   0,     0,     0,     0,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,
   0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,    -1,     0,     0,
   0,     0,     0,     0,     8,     8,     8,     8,     8,     8,     8,     8,     0,     0,     0,     0,
   0,     0,     0,     0,     8,     8,     8,     8,     8,     8,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     8,     8,     8,     8,     8,     8,     8,     8,     0,     0,     0,     0,
   0,     0,     0,     0,     8,     8,     8,     8,     8,     8,     8,     8,     0,     0,     0,     0,
   0,     0,     0,     0,     8,     8,     8,     8,     8,     8,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     8,     0,     8,     0,     8,     0,     8,     0,     0,     0,     0,
   0,     0,     0,     0,     8,     8,     8,     8,     8,     8,     8,     8,     0,     0,     0,     0,
   0,     0,     0,     0,     74,    74,    86,    86,    86,    86,    100,   100,   128,   128,   112,   112,
   126,   126,   0,     0,     8,     8,     8,     8,     8,     8,     8,     8,     0,     0,     0,     0,
   0,     0,     0,     0,     8,     8,     8,     8,     8,     8,     8,     8,     0,     0,     0,     0,
   0,     0,     0,     0,     8,     8,     8,     8,     8,     8,     8,     8,     0,     0,     0,     0,
   0,     0,     0,     0,     8,     8,     0,     9,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,    -7205,  0,     0,     0,     0,     9,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     8,     8,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     8,     8,     0,     0,     0,     7,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     9,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,    -16,   -16,   -16,   -16,   -16,   -16,   -16,   -16,
  -16,   -16,   -16,   -16,   -16,   -16,   -16,   -16,    0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,    -26,   -26,   -26,   -26,   -26,   -26,   -26,   -26,
  -26,   -26,   -26,   -26,   -26,   -26,   -26,   -26,   -26,   -26,   -26,   -26,   -26,   -26,   -26,   -26,
  -26,   -26,    0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,    -40,   -40,   -40,   -40,   -40,   -40,
  -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,
  -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,   -40,
  -40,   -40,    0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0
};


const unsigned short lower_plane[68]=
{
  0,    74,   74,   76,   203,  74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   74,   74,   74,   74,   74,
  74,   74,   74,   7
};


const unsigned short lower_block[331]=
{
  0,     91,    219,   346,   466,   517,   517,   639,
  767,   893,   1021,  517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   1149,  1255,  1376,  1496,
  517,   517,   1621,  517,   517,   517,   517,   517,
  517,   1733,  517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   1828,  517,   517,   517,   517,   517,
  517,   517,   517,   1956,  517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517,   517,   517,   517,   517,   517,
  517,   517,   517
};


const short lower_data[2084]=
{
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,
   32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     32,    32,    32,    32,    32,
   32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,
   32,    32,    0,     32,    32,    32,    32,    32,    32,    32,    0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,    -199,   0,     1,     0,     1,
   0,     1,     0,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,
   1,     0,     1,     0,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,    -121,   1,     0,     1,     0,     1,     0,     0,     210,   1,     0,     1,     0,
   206,   1,     0,     205,   205,   1,     0,     0,     79,    202,   203,   1,     0,     205,   207,   0,
   211,   209,   1,     0,     0,     0,     211,   213,   0,     214,   1,     0,     1,     0,     1,     0,
   218,   1,     0,     218,   0,     0,     1,     0,     218,   1,     0,     217,   217,   1,     0,     1,
   0,     219,   1,     0,     0,     0,     1,     0,     0,     0,     0,     0,     0,     0,     2,     1,
   0,     2,     1,     0,     2,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     0,     1,     0,     1,     0,     1,     0,     1,     0,
   1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     0,     2,     1,     0,     1,     0,
  -97,   -56,    1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,
   1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,
   1,     0,    -130,   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,
   1,     0,     1,     0,     1,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     38,    0,     37,    37,    37,    0,     64,    0,     63,    63,    0,
   32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,
   32,    0,     32,    32,    32,    32,    32,    32,    32,    32,    32,    0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     0,
   0,     0,     0,    -60,    0,     0,     1,     0,    -7,     1,     0,     0,     0,     0,     0,     80,
   80,    80,    80,    80,    80,    80,    80,    80,    80,    80,    80,    80,    80,    80,    80,    32,
   32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,
   32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     0,     1,     0,
   1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     0,     0,     1,     0,     0,     0,     0,     0,     0,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     48,    48,
   48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,
   48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,    48,
   48,    48,    48,    48,    0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,     0,     1,
   0,     0,     0,     0,     0,     0,     0,     0,    -8,    -8,    -8,    -8,    -8,    -8,    -8,    -8,
   0,     0,     0,     0,     0,     0,     0,     0,    -8,    -8,    -8,    -8,    -8,    -8,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,    -8,    -8,    -8,    -8,    -8,    -8,    -8,    -8,
   0,     0,     0,     0,     0,     0,     0,     0,    -8,    -8,    -8,    -8,    -8,    -8,    -8,    -8,
   0,     0,     0,     0,     0,     0,     0,     0,    -8,    -8,    -8,    -8,    -8,    -8,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,    -8,     0,    -8,     0,    -8,     0,    -8,
   0,     0,     0,     0,     0,     0,     0,     0,    -8,    -8,    -8,    -8,    -8,    -8,    -8,    -8,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
  -8,    -8,    -8,    -8,    -8,    -8,    -8,    -8,     0,     0,     0,     0,     0,     0,     0,     0,
  -8,    -8,    -8,    -8,    -8,    -8,    -8,    -8,     0,     0,     0,     0,     0,     0,     0,     0,
  -8,    -8,    -8,    -8,    -8,    -8,    -8,    -8,     0,     0,     0,     0,     0,     0,     0,     0,
  -8,    -8,    -74,   -74,   -9,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
  -86,   -86,   -86,   -86,   -9,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
  -8,    -8,    -100,  -100,   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
  -8,    -8,    -112,  -112,  -7,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
  -128,  -128,  -126,  -126,  -9,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,    -7517,  0,     0,     0,    -8383,
  -8262,  0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     16,    16,    16,    16,    16,    16,    16,    16,    16,    16,    16,
   16,    16,    16,    16,    16,    0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     26,    26,    26,    26,    26,
   26,    26,    26,    26,    26,    26,    26,    26,    26,    26,    26,    26,    26,    26,    26,    26,
   26,    26,    26,    26,    26,    0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,
   32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,    0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,
   40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,
   40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,    0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   0,     0,     0,     0
};

const std::ctype_base::mask digit = std::ctype_base::digit;
const std::ctype_base::mask xdigit = std::ctype_base::xdigit;
const std::ctype_base::mask cntrl = std::ctype_base::cntrl;
const std::ctype_base::mask space = std::ctype_base::space;
const std::ctype_base::mask upper = std::ctype_base::upper;
const std::ctype_base::mask lower = std::ctype_base::lower;
const std::ctype_base::mask alpha = std::ctype_base::alpha;
const std::ctype_base::mask punct = std::ctype_base::punct;
const std::ctype_base::mask print = std::ctype_base::print;
const std::ctype_base::mask alnum = std::ctype_base::alnum;
const std::ctype_base::mask graph = std::ctype_base::graph;

const std::ctype_base::mask ctype_data[13936]= 
{
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|space) & ~(print)),
    std::ctype_base::mask((cntrl|space) & ~(print)),
    std::ctype_base::mask((cntrl|space) & ~(print)),
    std::ctype_base::mask((cntrl|space) & ~(print)),
    std::ctype_base::mask((cntrl|space) & ~(print)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper|xdigit) & ~(digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper|xdigit) & ~(digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper|xdigit) & ~(digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper|xdigit) & ~(digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper|xdigit) & ~(digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper|xdigit) & ~(digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower|xdigit) & ~(digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|upper) & ~(xdigit|digit|lower|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|print|punct) & ~(xdigit|digit|upper|lower|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha|lower) & ~(xdigit|digit|upper|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|digit|xdigit) & ~(upper|lower|alpha|space|punct)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((graph|alnum|print|alpha) & ~(xdigit|digit|upper|lower|punct|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl|print|space) & ~(xdigit|digit|upper|lower|punct|alpha)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
    std::ctype_base::mask((cntrl) & ~(xdigit|digit|upper|lower|punct|alpha|space)),
 std::ctype_base::mask(0) };
}

uint32_t tolower(uint32_t ucs)
{
    return ucs + lower_data[lower_block[lower_plane[ucs>>14]+((ucs>>7)&127)]+(ucs&127)];
}


uint32_t toupper(uint32_t ucs)
{
    return ucs + upper_data[upper_block[upper_plane[ucs>>14]+((ucs>>7)&127)]+(ucs&127)];
}


std::ctype_base::mask ctypeMask(uint32_t ch)
{
    return ctype_data[ ctype_lookup2[ ctype_lookup1[ch>>14]+((ch>>7)&127) ]+(ch&127) ];
}

} // namespace zim
